Public MustInherit Class AsymCrypt
	Inherits Crypto

	Public Enum KeyOwner As Byte
		Company1 = 1
		Company2 = 2
	End Enum

	Public Enum KeyLocation As Byte
		File = 0
		Database = 1
	End Enum

	Public Enum KeySize As Integer
		_512 = 512
		_1024 = 1024
		_2048 = 2048
	End Enum

	Public Enum KeyType As Byte
		_Private = 1
		_Public = 2
	End Enum

	Private m_RSAKey As Cryptography.RSACryptoServiceProvider

	Public Sub New(ByVal p_KeySize As KeySize)
		Dim cspParms As Cryptography.CspParameters
		cspParms = New Cryptography.CspParameters()

		cspParms.Flags = Cryptography.CspProviderFlags.UseMachineKeyStore

		m_RSAKey = New System.Security.Cryptography.RSACryptoServiceProvider(p_KeySize, cspParms)

		cspParms = Nothing

	End Sub

	Protected Overrides Sub Finalize()
		m_RSAKey.Clear()
		m_RSAKey = Nothing

		MyBase.Finalize()

	End Sub

	Public ReadOnly Property RSAKey() As Cryptography.RSACryptoServiceProvider
		Get
			Return m_RSAKey
		End Get
	End Property

	Public Sub GetKey(ByVal p_KeyOwner As KeyOwner, _
	ByVal p_KeyType As KeyType, _
	ByVal p_KeyLocation As KeyLocation, _
	Optional ByVal p_KeyPath As String = "")

		Dim strKey As String = "vVWoYs8+/rUCJBb+kGN4LALyjrImQ18K"
		Dim strIV As String = "7lksYv8v8+4="

		Dim KeyDecrypt As SymDeCrypt
		KeyDecrypt = New SymDeCrypt(SymCrypt.CryptoAlgorithm.TripleDES, Security.Cryptography.CipherMode.CBC, Security.Cryptography.PaddingMode.PKCS7, SymCrypt.TripleDESKeySize._192)
		KeyDecrypt.Key = strKey
		KeyDecrypt.IV = strIV

		Try
			Select Case p_KeyLocation
				Case KeyLocation.File
					If p_KeyPath <> "" Then
						Dim fileRdr As System.IO.StreamReader
						fileRdr = New System.IO.StreamReader(p_KeyPath)

						m_RSAKey.FromXmlString(KeyDecrypt.Crypt(fileRdr.ReadToEnd))

						fileRdr.DiscardBufferedData()
						fileRdr.Close()

					Else
						Throw New Exception("AsymCrypt.GetKey (No Path Specified for Key File)")
					End If
				Case KeyLocation.Database
					Dim sqlConn As SqlClient.SqlConnection
					Dim strConnection As String
					Dim sqlCmd As SqlClient.SqlCommand
					Dim sqlRdr As SqlClient.SqlDataReader

					strConnection = "Database Connection String Here"

					sqlConn = New SqlClient.SqlConnection(strConnection)

					sqlCmd = New SqlClient.SqlCommand()
					sqlCmd.CommandType = CommandType.StoredProcedure
					sqlCmd.CommandText = "GetKey"
					sqlCmd.Connection = sqlConn

					sqlCmd.Parameters.Add("@KeyType", p_KeyType)
					sqlCmd.Parameters.Add("@KeyOwner", p_KeyOwner)
					sqlCmd.Parameters.Add("@KeyLength", m_RSAKey.KeySize)

					sqlConn.Open()
					sqlRdr = sqlCmd.ExecuteReader(CommandBehavior.CloseConnection)

					If sqlRdr.Read() Then

						m_RSAKey.FromXmlString(KeyDecrypt.Crypt(sqlRdr.GetString(0)))

					Else

						Throw New Exception("AsymCrypt.GetString (Key not found in database.)")

					End If

					sqlCmd.Dispose()
					sqlRdr.Close()
					sqlRdr = Nothing
					sqlConn.Dispose()
			End Select

		Catch e As Exception

			Throw e

		Catch sqle As SqlClient.SqlException

			Throw New Exception(sqle.Message)

		Finally

			KeyDecrypt = Nothing

		End Try

	End Sub

	Public Sub SaveKey(ByVal p_KeyOwner As KeyOwner, _
	ByVal p_KeyType As KeyType, _
	ByVal p_KeyLocation As KeyLocation, _
	Optional ByVal p_KeyPath As String = "")

		Dim strKey As String = "vVWoYs8+/rUCJBb+kGN4LALyjrImQ18K"
		Dim strIV As String = "7lksYv8v8+4="

		Dim KeyEncrypt As SymEnCrypt
		KeyEncrypt = New SymEnCrypt(SymCrypt.CryptoAlgorithm.TripleDES, Security.Cryptography.CipherMode.CBC, Security.Cryptography.PaddingMode.PKCS7, SymCrypt.TripleDESKeySize._192)
		KeyEncrypt.Key = strKey
		KeyEncrypt.IV = strIV

		Try
			Select Case p_KeyLocation
				Case KeyLocation.File
					If p_KeyPath <> "" Then
						Dim fileKeyWriter As IO.StreamWriter

						fileKeyWriter = New IO.StreamWriter(p_KeyPath)

						fileKeyWriter.Write(KeyEncrypt.Crypt(m_RSAKey.ToXmlString((p_KeyType = KeyType._Private))))

						fileKeyWriter.Flush()
						fileKeyWriter.Close()

					Else
						Throw New Exception("AsymCrypt.SaveKey (No Path Specified for Key File)")
					End If
				Case KeyLocation.Database
					Dim sqlConn As SqlClient.SqlConnection
					Dim strConnection As String
					Dim sqlCmd As SqlClient.SqlCommand

					strConnection = "Database Connection String Here"

					sqlConn = New SqlClient.SqlConnection(strConnection)

					sqlCmd = New SqlClient.SqlCommand()
					sqlCmd.CommandType = CommandType.StoredProcedure
					sqlCmd.CommandText = "SaveKey"
					sqlCmd.Connection = sqlConn

					sqlCmd.Parameters.Add("@KeyType", p_KeyType)
					sqlCmd.Parameters.Add("@KeyOwner", p_KeyOwner)
					sqlCmd.Parameters.Add("@KeyLength", m_RSAKey.KeySize)
					sqlCmd.Parameters.Add("@KeyXML", KeyEncrypt.Crypt(m_RSAKey.ToXmlString((p_KeyType = KeyType._Private))))

					sqlConn.Open()

					sqlCmd.ExecuteNonQuery()

					sqlCmd.Dispose()
					sqlConn.Close()
					sqlConn.Dispose()
			End Select

		Catch e As Exception

			Throw e

		Catch sqle As SqlClient.SqlException

			Throw New Exception(sqle.Message)

		Finally

			KeyEncrypt = Nothing

		End Try

	End Sub


End Class
