Public MustInherit Class SymCrypt
	Inherits Crypto

	Public Enum DESKeySize As Integer
		_64 = 64
	End Enum

	Public Enum RC2KeySize As Integer
		_40 = 40
		_48 = 48
		_56 = 56
		_64 = 64
		_72 = 72
		_80 = 80
		_88 = 88
		_96 = 96
		_104 = 104
		_112 = 112
		_120 = 120
		_128 = 128
	End Enum

	Public Enum RijndaelKeySize As Integer
		_128 = 128
		_192 = 192
		_256 = 256
	End Enum

	Public Enum TripleDESKeySize As Integer
		_128 = 128
		_192 = 192
	End Enum

	Public Enum CryptoAlgorithm As Byte
		DES = 0
		TripleDES = 1
		RC2 = 2
		RijndaelManaged = 3
	End Enum

	Private m_Key As Cryptography.SymmetricAlgorithm

	Public Sub New(ByVal p_KeyAlgorithm As CryptoAlgorithm, _
	ByVal p_CipherMode As Cryptography.CipherMode, _
	ByVal p_PaddingMode As Cryptography.PaddingMode, _
	ByVal p_KeySize As DESKeySize)

		If p_KeyAlgorithm <> CryptoAlgorithm.DES Then
			Throw New Exception("KeySize selection does not match algorithm selection")
		End If

		Call Construct(p_KeyAlgorithm, p_CipherMode, p_PaddingMode, p_KeySize)

	End Sub

	Public Sub New(ByVal p_KeyAlgorithm As CryptoAlgorithm, _
	ByVal p_CipherMode As Cryptography.CipherMode, _
	ByVal p_PaddingMode As Cryptography.PaddingMode, _
	ByVal p_KeySize As RC2KeySize)

		If p_KeyAlgorithm <> CryptoAlgorithm.RC2 Then
			Throw New Exception("KeySize selection does not match algorithm selection")
		End If

		Call Construct(p_KeyAlgorithm, p_CipherMode, p_PaddingMode, p_KeySize)

	End Sub

	Public Sub New(ByVal p_KeyAlgorithm As CryptoAlgorithm, _
	ByVal p_CipherMode As Cryptography.CipherMode, _
	ByVal p_PaddingMode As Cryptography.PaddingMode, _
	ByVal p_KeySize As RijndaelKeySize)

		If p_KeyAlgorithm <> CryptoAlgorithm.RijndaelManaged Then
			Throw New Exception("KeySize selection does not match algorithm selection")
		End If

		Call Construct(p_KeyAlgorithm, p_CipherMode, p_PaddingMode, p_KeySize)

	End Sub

	Public Sub New(ByVal p_KeyAlgorithm As CryptoAlgorithm, _
	ByVal p_CipherMode As Cryptography.CipherMode, _
	ByVal p_PaddingMode As Cryptography.PaddingMode, _
	ByVal p_KeySize As TripleDESKeySize)

		If p_KeyAlgorithm <> CryptoAlgorithm.TripleDES Then
			Throw New Exception("KeySize selection does not match algorithm selection")
		End If

		Call Construct(p_KeyAlgorithm, p_CipherMode, p_PaddingMode, p_KeySize)

	End Sub

	Protected Overrides Sub Finalize()
		m_Key.Clear()
		m_Key = Nothing

		MyBase.Finalize()

	End Sub

	Public Sub Construct(ByVal p_KeyAlgorithm As CryptoAlgorithm, _
	ByVal p_CipherMode As Cryptography.CipherMode, _
	ByVal p_PaddingMode As Cryptography.PaddingMode, _
	ByVal p_KeySize As Integer)

		Select Case p_KeyAlgorithm

			Case CryptoAlgorithm.DES
				m_Key = New Cryptography.DESCryptoServiceProvider()
			Case CryptoAlgorithm.RC2
				m_Key = New Cryptography.RC2CryptoServiceProvider()
			Case CryptoAlgorithm.RijndaelManaged
				m_Key = New Cryptography.RijndaelManaged()
			Case CryptoAlgorithm.TripleDES
				m_Key = New Cryptography.TripleDESCryptoServiceProvider()

		End Select

		m_Key.Mode = p_CipherMode
		m_Key.Padding = p_PaddingMode
		m_Key.KeySize = p_KeySize

	End Sub

	Public Property Key() As String
		Get
			Return Convert.ToBase64String(m_Key.Key)
		End Get
		Set(ByVal Value As String)
			m_Key.Key = Convert.FromBase64String(Value)
		End Set
	End Property

	Public Property IV() As String
		Get
			Return Convert.ToBase64String(m_Key.IV)
		End Get
		Set(ByVal Value As String)
			If Value = "" Then
				m_Key.IV = Array.CreateInstance(GetType(Byte), 1)
			Else
				m_Key.IV = Convert.FromBase64String(Value)
			End If
		End Set
	End Property

	Public Property CryptoKey() As Cryptography.SymmetricAlgorithm
		Get
			Return m_Key
		End Get
		Set(ByVal Value As Cryptography.SymmetricAlgorithm)
			m_Key = Value
		End Set
	End Property


End Class
